/**
 * Copyright (c) 2015 Ivan Bausov <ivan.bausov@gmail.com> (MIT Licensed)
 * A part of B:STRUCT package <https://github.com/ivan-bausov/bstruct>
 */
var _ = require('underscore');
var enums = require('./compiler.e');
var TYPES = enums.TYPES;
var Placeholders = (function () {
    function Placeholders() {
    }
    Placeholders.CHILDREN = '{CHILDREN}';
    Placeholders.BLOCK = '{BLOCK}';
    Placeholders.ATTRIBUTES = '{ATTRIBUTES}';
    Placeholders.TAG = '{TAG}';
    return Placeholders;
})();
var HtmlItem = (function () {
    function HtmlItem(data) {
        this.data = data;
    }
    HtmlItem.isAttributeMultivalued = function (name) {
        var multivalued_attributes = {
            'class': true
        };
        return !!multivalued_attributes[name];
    };
    HtmlItem.prototype.getName = function () {
        var data = this.data;
        switch (data && data.type) {
            case TYPES.BLOCK:
                return HtmlItem.getBlockName(data.name);
            case TYPES.ELEMENT:
                return HtmlItem.getElementName(data.name);
            default:
                return null;
        }
    };
    HtmlItem.prototype.getHTML = function () {
        var attributes_string, item_template, item_html, html;
        if (this.data) {
            item_template = HtmlItem.TEMPLATES[this.data.tag] || HtmlItem.TEMPLATES.ANY;
            attributes_string = this.getAttributesString(item_template);
            item_html = item_template.template
                .replace(Placeholders.ATTRIBUTES, attributes_string ? ' ' + attributes_string : attributes_string)
                .replace(new RegExp(Placeholders.TAG, 'g'), this.data.tag || 'div');
            if (this.data.count) {
                var items = [];
                for (var i = this.data.count; i; i--) {
                    items.push(item_html);
                }
                html = items.join('\n');
            }
            return html || item_html;
        }
        else {
            return HtmlItem.TEMPLATES.EMPTY.template;
        }
    };
    HtmlItem.prototype.getAttributesString = function (template) {
        var attributes_string, data = this.data, attributes = template.attributes || [], attributes_value_dictionary = {};
        if (data) {
            attributes = _.union(attributes, data.attributes || []);
            name = this.getName();
            if (name) {
                attributes.unshift({
                    name: 'class',
                    value: name
                });
            }
            _.each(attributes, function (attribute) {
                attributes_value_dictionary[attribute.name] = attributes_value_dictionary[attribute.name] && HtmlItem.isAttributeMultivalued(attribute.name) ?
                    attributes_value_dictionary[attribute.name] + ' ' + attribute.value :
                    attribute.value;
            });
            attributes = [];
            _.each(attributes_value_dictionary, function (value, name) {
                attributes.push({
                    name: name,
                    value: value
                });
            });
        }
        attributes_string = _.map(attributes, function (attribute) {
            return attribute.name + '="' + attribute.value + '"';
        }).join(' ') || '';
        return attributes_string;
    };
    HtmlItem.getBlockName = function (name) {
        return 'block-' + name;
    };
    HtmlItem.getElementName = function (name) {
        return name ? Placeholders.BLOCK + '_' + name : null;
    };
    HtmlItem.TEMPLATES = {
        'a': {
            template: '<a' + Placeholders.ATTRIBUTES + '>' + Placeholders.CHILDREN + '</a>',
            attributes: [
                {
                    name: 'href',
                    value: '#'
                },
                {
                    name: 'title',
                    value: ''
                }
            ]
        },
        'img': {
            template: '<img' + Placeholders.ATTRIBUTES + '/>',
            attributes: [
                {
                    name: 'src',
                    value: ''
                },
                {
                    name: 'alt',
                    value: ''
                }
            ]
        },
        ANY: {
            template: '<' + Placeholders.TAG + Placeholders.ATTRIBUTES + '>' + Placeholders.CHILDREN + '</' + Placeholders.TAG + '>'
        },
        EMPTY: {
            template: Placeholders.CHILDREN
        }
    };
    return HtmlItem;
})();
var PlaceholderItem = (function () {
    function PlaceholderItem(item) {
        this.item = item;
    }
    PlaceholderItem.prototype.getHTML = function () {
        var item_html;
        item_html = (this.item.children.length ?
            PlaceholderItem.TEMPLATES.PLACEHOLDER_WRAPPER :
            PlaceholderItem.TEMPLATES.PLACEHOLDER)
            .replace(new RegExp(Placeholders.TAG, 'g'), this.item.data.name);
        return item_html;
    };
    PlaceholderItem.TEMPLATES = {
        PLACEHOLDER_WRAPPER: '{{#' + Placeholders.TAG + '}}' + Placeholders.CHILDREN + '{{/' + Placeholders.TAG + '}}',
        PLACEHOLDER: '{{' + Placeholders.TAG + '}}'
    };
    return PlaceholderItem;
})();
var Compiler = (function () {
    function Compiler(data) {
        this.html = Compiler.render(data).replace(new RegExp(Placeholders.BLOCK + '_', 'g'), '');
    }
    Compiler.prototype.compile = function () {
        return this.html;
    };
    Compiler.render = function (item) {
        var item_code = Compiler.renderItem(item), children_code = Compiler.renderChildren(item);
        return Compiler.applyChildren(item_code, children_code);
    };
    Compiler.renderItem = function (item) {
        if (item.data && item.data.type === TYPES.PLACEHOLDER) {
            return new PlaceholderItem(item).getHTML();
        }
        else {
            return new HtmlItem(item.data).getHTML();
        }
    };
    Compiler.applyChildren = function (item_code, children_code) {
        if (children_code && item_code !== HtmlItem.TEMPLATES.EMPTY.template) {
            children_code = '\n' + children_code + '\n';
        }
        return item_code.replace(new RegExp(Placeholders.CHILDREN, 'g'), children_code);
    };
    Compiler.mergeChildren = function (children, item) {
        var result;
        if (item.data) {
            result = _.map(children.join('\n').split('\n'), function (str) {
                return str && ('    ' + str);
            }).join('\n');
            if (item.data.type === TYPES.BLOCK) {
                result = result.replace(new RegExp(Placeholders.BLOCK, 'g'), item.data.name);
            }
        }
        else {
            result = children.join('\n');
        }
        return result;
    };
    Compiler.renderChildren = function (item) {
        return Compiler.mergeChildren(_.map(item.children, function (child) {
            return Compiler.render(child);
        }), item);
    };
    return Compiler;
})();
module.exports = Compiler;
