/**
 * Copyright (c) 2015 Ivan Bausov <ivan.bausov@gmail.com> (MIT Licensed)
 * A part of B:STRUCT package <https://github.com/ivan-bausov/bstruct>
 */
var _ = require('underscore');
var enums = require('./compiler.e');
var Tree = require('./tree.t');
var TYPES = enums.TYPES;
;
var Compiler = (function () {
    function Compiler(data) {
        this.source_strings = [];
        this.tree = new Tree();
        this.source_strings = data.split('\n');
        this.buildTree();
    }
    Compiler.prototype.compile = function () {
        return this.tree.serialize();
    };
    Compiler.prototype.buildTree = function () {
        var _this = this;
        _.each(this.source_strings, function (line, index) {
            var declaration_selected = null, declaration, max, i;
            if (!line.trim()) {
                return;
            }
            for (i = 0, max = Compiler.declarations.length; i < max; i++) {
                declaration = Compiler.declarations[i];
                if (declaration.pattern.test(line)) {
                    declaration_selected = declaration;
                    break;
                }
            }
            if (declaration_selected) {
                _this.tree.upTo(Compiler.parseLevel(line));
                if (declaration_selected.type === TYPES.ATTRIBUTE) {
                    var info = _this.tree.get().getInfo();
                    if (!info.attributes) {
                        info.attributes = [];
                    }
                    info.attributes.push(declaration_selected.parser(line, index));
                }
                else {
                    _this.tree.add(declaration_selected.parser(line, index));
                }
            }
            else {
                throw new Error('Unnable to parse code at line: ' + index + ':' + line);
            }
        });
    };
    Compiler.prototype.getSourceStrings = function () {
        return this.source_strings;
    };
    Compiler.parseLevel = function (str) {
        var matches = str.match(/^(\s*)/), space_length = matches ? matches[1] && matches[1].length : 0, delta = space_length % 4, count = Math.floor(space_length / 4);
        return delta ? count + 1 : count;
    };
    Compiler.parseBlockDeclaration = function (line, index) {
        return Compiler.parseDeclaration(line, /^\s*b\:\s*([^>^\*\s]+)(:?\s*>\s*([^\*\s]+))?\s*(:?\s*\*\s*(\d+))?\s*$/, TYPES.BLOCK, Compiler.Errors.BLOCK_DECLARATION_SYNTAX_ERROR, index);
    };
    Compiler.parseElementDeclaration = function (line, index) {
        return Compiler.parseDeclaration(line, /^\s*e\:\s*([^>^\*\s]+)?(:?\s*>\s*([^\*\s]+))?\s*(:?\s*\*\s*(\d+))?\s*$/, TYPES.ELEMENT, Compiler.Errors.ELEMENT_DECLARATION_SYNTAX_ERROR, index);
    };
    Compiler.parsePlaceholderDeclaration = function (line, index) {
        return Compiler.parseDeclaration(line, /^\s*p\:\s*(\S+)?\s*$/, TYPES.PLACEHOLDER, Compiler.Errors.PLACEHOLDER_DECLARATION_SYNTAX_ERROR, index);
    };
    Compiler.parseAttributeDeclaration = function (line, index) {
        var matches = line.match(/^\s*(\S+)\s*\:\s*(\S+)\s*$/);
        if (matches) {
            return {
                name: matches[1],
                value: matches[2]
            };
        }
        else {
            throw Compiler.createError(Compiler.Errors.ATTRIBUTE_DECLARATION_SYNTAX_ERROR, line, index);
        }
    };
    Compiler.parseDeclaration = function (line, pattern, item_type, error_message, index) {
        var matches = line.match(pattern), item_data = null;
        if (matches) {
            item_data = {
                type: item_type,
                name: matches[1] || null,
                tag: matches[3] || null
            };
            matches[5] && (item_data.count = parseInt(matches[5]));
        }
        else {
            throw Compiler.createError(error_message, line, index);
        }
        return item_data;
    };
    Compiler.createError = function (message, line, line_number) {
        return new Error(message + ' at line:' + line_number + ':' + line);
    };
    Compiler.Errors = {
        BLOCK_DECLARATION_SYNTAX_ERROR: "BLOCK declaration syntax error",
        ELEMENT_DECLARATION_SYNTAX_ERROR: "ELEMENT declaration syntax error",
        ATTRIBUTE_DECLARATION_SYNTAX_ERROR: "ATTRIBUTE declaration syntax error",
        PLACEHOLDER_DECLARATION_SYNTAX_ERROR: "PLACEHOLDER declaration syntax error"
    };
    Compiler.declarations = [
        {
            type: TYPES.BLOCK,
            pattern: /^(:?\s)*b\:.+$/,
            parser: Compiler.parseBlockDeclaration
        },
        {
            type: TYPES.ELEMENT,
            pattern: /^(:?\s)*e\:.*$/,
            parser: Compiler.parseElementDeclaration
        },
        {
            type: TYPES.PLACEHOLDER,
            pattern: /^(:?\s)*p\:.+$/,
            parser: Compiler.parsePlaceholderDeclaration
        },
        {
            type: TYPES.ATTRIBUTE,
            pattern: /^\s*(\S+)\s*\:\s*(\S+)\s*$/,
            parser: Compiler.parseAttributeDeclaration
        }
    ];
    return Compiler;
})();
module.exports = Compiler;
